"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var humanize_duration_lang_1 = require("./humanize-duration.lang");
require("./lang/en");
var HumanizeDuration = /** @class */ (function () {
    function HumanizeDuration(languageUtil) {
        this.languageUtil = languageUtil;
        this.defaultOptions = {
            language: 'en',
            delimiter: ', ',
            spacer: ' ',
            conjunction: '',
            serialComma: true,
            units: ['y', 'mo', 'w', 'd', 'h', 'm', 's'],
            languages: {},
            largest: 10,
            decimal: '.',
            round: false,
            unitMeasures: {
                y: 31557600000,
                mo: 2629800000,
                w: 604800000,
                d: 86400000,
                h: 3600000,
                m: 60000,
                s: 1000,
                ms: 1
            }
        };
        this.options = undefined;
        if (this.languageUtil === undefined) {
            this.languageUtil = humanize_duration_lang_1.LANGUAGES;
        }
        this.options = this.defaultOptions;
    }
    HumanizeDuration.prototype.humanize = function (value, tempOptions) {
        var options = tempOptions !== undefined ? this.extend(this.defaultOptions, tempOptions) : this.defaultOptions;
        return this.doHumanization(value, options);
    };
    HumanizeDuration.prototype.setOptions = function (passedOptions) {
        this.options = passedOptions !== undefined ? this.extend(this.defaultOptions, passedOptions) : this.defaultOptions;
    };
    HumanizeDuration.prototype.getSupportedLanguages = function () {
        var result = [];
        for (var language in this.languageUtil.languages) {
            if (this.languageUtil.languages.hasOwnProperty(language)) {
                result.push(language);
            }
        }
        return result;
    };
    HumanizeDuration.prototype.addLanguage = function (key, lang) {
        this.languageUtil.addLanguage(key, lang);
    };
    HumanizeDuration.prototype.doHumanization = function (ms, options) {
        var i, len, piece;
        // Make sure we have a positive number.
        // Has the nice sideffect of turning Number objects into primitives.
        ms = Math.abs(ms);
        var dictionary = options.languages[options.language] || this.languageUtil.languages[options.language];
        if (!dictionary) {
            throw new Error('No language ' + dictionary + '.');
        }
        var pieces = [];
        // Start at the top and keep removing units, bit by bit.
        var unitName;
        var unitMS, unitCount;
        for (i = 0, len = options.units.length; i < len; i++) {
            unitName = options.units[i];
            unitMS = options.unitMeasures[unitName];
            // What's the number of full units we can fit?
            if (i + 1 === len) {
                unitCount = ms / unitMS;
            }
            else {
                unitCount = Math.floor(ms / unitMS);
            }
            // Add the string.
            pieces.push({
                unitCount: unitCount,
                unitName: unitName
            });
            // Remove what we just figured out.
            ms -= unitCount * unitMS;
        }
        var firstOccupiedUnitIndex = 0;
        for (i = 0; i < pieces.length; i++) {
            if (pieces[i].unitCount) {
                firstOccupiedUnitIndex = i;
                break;
            }
        }
        if (options.round) {
            var ratioToLargerUnit = void 0, previousPiece = void 0;
            for (i = pieces.length - 1; i >= 0; i--) {
                piece = pieces[i];
                piece.unitCount = Math.round(piece.unitCount);
                if (i === 0) {
                    break;
                }
                ;
                previousPiece = pieces[i - 1];
                ratioToLargerUnit = options.unitMeasures[previousPiece.unitName] / options.unitMeasures[piece.unitName];
                if ((piece.unitCount % ratioToLargerUnit) === 0
                    || (options.largest && ((options.largest - 1) < (i - firstOccupiedUnitIndex)))) {
                    previousPiece.unitCount += piece.unitCount / ratioToLargerUnit;
                    piece.unitCount = 0;
                }
            }
        }
        var result = [];
        for (i = 0, pieces.length; i < len; i++) {
            piece = pieces[i];
            if (piece.unitCount) {
                result.push(this.render(piece.unitCount, piece.unitName, dictionary, options));
            }
            if (result.length === options.largest) {
                break;
            }
            ;
        }
        if (result.length) {
            if (!options.conjunction || result.length === 1) {
                return result.join(options.delimiter);
            }
            else if (result.length === 2) {
                return result.join(options.conjunction);
            }
            else if (result.length > 2) {
                return result.slice(0, -1).join(options.delimiter) + (options.serialComma ? ',' : '')
                    + options.conjunction + result.slice(-1);
            }
        }
        else {
            return this.render(0, options.units[options.units.length - 1], dictionary, options);
        }
    };
    HumanizeDuration.prototype.render = function (count, type, dictionary, options) {
        var decimal;
        if (options.decimal === void 0) {
            decimal = dictionary.decimal;
        }
        else {
            decimal = options.decimal;
        }
        var countStr = count.toString().replace('.', decimal.toString());
        var dictionaryValue = dictionary[type];
        var word;
        if (typeof dictionaryValue === 'function') {
            word = dictionaryValue(count);
        }
        else {
            word = dictionaryValue;
        }
        return countStr + options.spacer + word;
    };
    HumanizeDuration.prototype.extend = function (options, override) {
        for (var prop in override) {
            if (options.hasOwnProperty(prop)) {
                options[prop] = override[prop];
            }
        }
        return options;
    };
    return HumanizeDuration;
}());
exports.HumanizeDuration = HumanizeDuration;
